import sys

# Define a utility method to clear the screen.
# This uses "ANSI Escape Sequences" - special combinations of
# characters that are understood by the terminal, but are difficult
# (or impossible) to type on a keyboard. `\033[` is the
# "escape sequence"; the characters after the escape sequnce are
# interpreted as commands. In this case, we're using:
#   2J - clear screen; and
#   H - move the cursor to the Home position (top left corner)
#
# For more details about ANSI escape sequences, see:
#     https://en.wikipedia.org/wiki/ANSI_escape_code
#
def clear_screen():
    print('\033[2J\033[H', end='')
    sys.stdout.flush()


# Use the ?25l ANSI escape code to hide the cursor, and ?12h to turn off
# local echo. This means we won't be able to see what keys we type.
def hide_cursor():
    print('\033[?25l\033[?12h')
    sys.stdout.flush()


# Use the ?25h ANSI escape code to restore the cursor,
# and ?12l to turn on local echo
def show_cursor():
    print('\033[?25h\033[?12l')
    sys.stdout.flush()



# Import the 'random' module, so you can generate random numbers
import random

print('Starship Take-off')
print('-----------------')
print()

# Select 2 random numbers. `random.randrange(X)` picks a number
# between 0 and X, *including* 0, but *not* including X. Adding
# 1 means we get a number between 1 and X, *inclusive*.
gravity = random.randrange(20) + 1
weight = random.randrange(40) + 1

# Compute the required launch force.
required = gravity * weight

# Print the gravity for the planet
print('Gravity =', gravity)

# Start a loop that will run until either there have been
# 10 attempts, or the user entered the correct force.
success = False
c = 0
while c < 10 and not success:
    # On each loop, increment the count of attempts,
    # and prompt the user for a value
    c = c + 1
    force = int(input("Force:"))

    # Compare the value entered with the required value.
    # If they match, set the success flag; otherwise,
    # print an error
    if force > required:
        print('Too high!')
    elif force < required:
        print('Too low!')
    else:
        success = True

    # As long as this isn't the last loop,
    # we can try again
    if c < 10:
        print('Try again.')

print()

# We've either been successful, or we've done 10 loops.
# Tell the user if they've been successful.
if success:
    print('Good take off!!')
else:
    print('You failed - the aliens got you')



# import randint, so we can get a random integer
# import atan and pi, so we can calculate the correct angle
# import sqrt, so we can calculate the correct speed
from random import randint
from math import atan, pi
from math import sqrt

# Set up the number of guesses we're going to get
guesses = 8

# Create a flag to keep track if we've won
won_game = False

print("INTERGALACTIC GAMES")

# Chooses the height to which you must launch your satellite,
# puts the value in height, and prints it.
height = randint(1, 100)
print("You must launch a satellite to a height of", height)

# As long as we have guesses left and haven't won yet, we'll keep trying
while guesses:
    guesses -= 1

    # Asks you for an angle and puts it in angle
    angle = int(input("Enter an angle (0-90): "))

    # Asks you for a speed and puts it in speed
    speed = int(input("Enter a speed (0-40000): "))

    # Subtracts the correct angle from your guess to determine
    # how close you were and stores the value in angle_diff
    angle_diff = angle - (atan(height / 3) * 180 / pi)

    # Subtracts the correct speed from your guess to determine
    # how close you were and stores the value in speed_diff
    speed_diff = speed - (3000 * sqrt(height + 1 / height))

    # Checks if you were close enough to win
    if abs(angle_diff) < 2 and abs(speed_diff) < 100:
        won_game = True
        break

    # Prints appropriate messages to help you with your next guess
    if angle_diff < -2:
        print("Too shallow.")
    elif angle_diff > 2:
        print("Too steep.")

    if speed_diff < -100:
        print("Too slow.")
    elif speed_diff > 100:
        print("Too fast.")
    print()

if won_game:
    print("You've done it!")
    print("NCTV WINS--thanks to you!")
else:
    print("You've failed.")
    print("You're fired!")



# Import 'random' so we can generate random numbers.
# The 'string' module gives us access to string-related utilities
# The 'time' module helps us to wait for some time
import random
import string
import time

print('VITAL MESSAGE')

# Variable to store the difficulty i.e length of string
difficulty = 0

# Loop until the right difficulty level is enterred
while difficulty < 4 or difficulty > 10:
    print('How difficult do you want the game to be? (4-10)')
    x = int(input())
    difficulty = x

# Create the secret message
message = ''.join(
    random.choice(string.ascii_letters)
    for i in range(0, difficulty)
)

print('Send this message:')
print(message)

# Display the secret message in accordance with the difficulty level
time.sleep(0.5 * difficulty)

code = input('Enter the secret code that need to be transferred')

if code == message:
    print('MESSAGE CORRECT')
    print('The war is over!')
else:
    print('YOU GOT IT WRONG')
    print('You should have sent:', message)



# Import `random` so we can generate random numbers.
# The `string` module gives us access to string-related utilities,
# like a list of lowercase characters.
import random
import string

print('ROBOT MISSILE')
print()
print('Type the correct code letter (A-Z) to defuse the missile.')
print('You have 4 chances.')
print()

# Choose, at random, a lowercase ascii character
code = random.choice(string.ascii_lowercase)

# Loop until we get a match, or we've tried 4 times
guesses = 0
success = False

while not success and guesses < 4:
    # Read a character from the user, and convert it to lower case
    guess = input('What is your guess? ').lower()
    guesses += 1

    # Check to see if the guess is correct. If it is, flag the success;
    # otherwise, print an error message hinting towards the final answer.
    if guess == code:
        success = True
    elif guess < code:
        print('The code is later than %s in the alphabet' % guess)
    elif guess > code:
        print('The code is earlier than %s in the alphabet' % guess)

# The loop has exited. Let the user know if they succeeded or not.
if success:
    print('TICK... FIZZ... CLICK...')
    print('You did it!')
else:
    print('BOOOOOOOOMMM...')
    print('You blew it!')
    print()
    print("The correct code was '%s'." % code)



# Import:
# * the `random` module so we can generate random numbers,
# * `select`, `sys`, `termios` and `tty` to manipulate the terminal
# * `time` so we can sleep
import random
import select
import sys
import termios
import time
import tty


# Normal terminals are defined a lot like typewriters. They are "line based" -
# they wait for the user to type a full line, then type Enter, and the entire
# line of input is processed. To be able to read a single character (without
# typing Enter), you need to put the terminal (or tty - short for TeleTYpe)
# into "raw" mode.
#
# However, putting a terminal into "raw" mode leaves the terminal in an unusual
# state, so it's good to clean up after yourself. We use the termios module
# to record the initial state of the terminal (using `tcgetattr`), and restore
# the state of the terminal (using `tcsetattr`) after you're done.

try:
    # sys.stdin is the "standard input" pipe - that is, the keyboard.
    fd = sys.stdin.fileno()

    # Record the initial settings of the input pipe
    old_settings = termios.tcgetattr(fd)

    # Set the input pipe to be in raw mode
    tty.setraw(fd)

    # Define a utility method to clear the screen.
    # This uses "ANSI Escape Sequences" - special combinations of
    # characters that are understood by the terminal, but are difficult
    # (or impossible) to type on a keyboard. `\033[` is the
    # "escape sequence"; the characters after the escape sequnce are
    # interpreted as commands. In this case, we're using:
    #   2J - clear screen; and
    #   H - move the cursor to the Home position (top left corner)
    #
    # For more details about ANSI escape sequences, see:
    #     https://en.wikipedia.org/wiki/ANSI_escape_code
    #
    def clear_screen():
        print('\033[2J\033[H', end='')
        sys.stdout.flush()

    # Another utility method - this time, to get a single character from
    # the keyboard.
    #
    # But though we're in "raw" mode, we can't answer the question "has the
    # user typed
    # key or not?" To do *that*, you need to use an operating system call
    # called `select`. Select is a method used to inspect the status of
    # "pipes" of content to determine if any content is ready. The keyboard
    # in an "input pipe"; the screen is an "output pipe". Select waits until
    # a specific pipe has content available; you also provide a timeout which
    # is the maximum length of time that the select call will wait until
    # content is available. This timeout can be "0 seconds", or it can be
    # as long as you're willing to wait.
    def getch(timeout=None):
        # If we aren't using a timeout, we can just read from the input pipe.
        # If we *do* have a timeout, use the select call to wait until input
        # is availble, or the timeout expires.
        if timeout is None or select.select([sys.stdin], [], [], timeout)[0]:
            ch = sys.stdin.read(1)
        else:
            ch = None

        # Return the character that was read
        # - or None, if no character was pressed.
        return ch

    # Now we can start the actual game.

    # Use the ?25l ANSI escape code to hide the cursor, and ?12h to turn off
    # local echo. This means we won't be able to see what keys we type.
    print('\033[?25l\033[?12h')

    score = 0

    # Do 10 iterations of showing bug eyes
    for t in range(10):
        clear_screen()

        # random.random() returns a floating point number in the range 0 -> 1
        # Add 0.5 to get a number between 0.5 and 1.5. That's how long we'll
        # sleep before displaying some eyes.
        time.sleep(random.random() + 0.5)

        # Pick a random zone in which to display the eyes.
        zone = random.randrange(4) + 1

        # The zone gives us the x,y offset
        if zone == 1:
            x_offset = 0
            y_offset = 0
        elif zone == 2:
            x_offset = 38
            y_offset = 0
        elif zone == 3:
            x_offset = 0
            y_offset = 12
        elif zone == 4:
            x_offset = 38
            y_offset = 12

        # Combine the offsets and a random number to postion the cursor
        # and print eyes. We can just print newlines to set the y (vertical)
        # position, then print spaces to set the x (horizontal) position.
        for y in range(random.randrange(12) + y_offset):
            print()

        print(' ' * (random.randrange(38) + x_offset) + '* *')

        # Wait up to a second for the player to respond.
        try:
            selected = int(getch(timeout=1))
        except TypeError:
            # If the user doesn't press a character, or they press something
            # other than a number, the call to int() will raise a ValueError.
            # Regardless, we know they didn't pick a zone; set the zone to None
            selected = None

        if selected == zone:
            score = score + 1

    # Clear the screen
    clear_screen()

finally:
    # No matter what happens, clean up after yourself.

    # Use the ?25h ANSI escape code to restore the cursor,
    # and ?12l to turn on local echo
    print('\033[?25h\033[?12l')

    # No matter what happens, drain the input pipe, and restore to
    # it's original state.
    termios.tcsetattr(fd, termios.TCSADRAIN, old_settings)

# Last of all, print the user's score.
# We do this *after* restoring the terminal settings so that the output
# behaves the way we'd expect a normal terminal to behave.
print('You blasted %s bugs' % score)



# Import 'random' so we can generate random numbers.
import random

# Initialy the score is 300
score = 300

# List of comments for making the game a more fun!
comments = [
    '** MICROS RULE! **',
    '*PEOPLE ARE SUPID*',
    '+A ROBOT FOR PRESIDENT!',
    '!COMPUTERS ARE GREAT!',
    'I AM BETTER THAN YOU!',
]

# Print a heading
print('  Number   Score')
print('~~~~~~~~~~~~~~~~')

# Loop while the score is not above 500 i.e you have won,
# or less than 0 means you have lost the game
while score > 0 and score < 500:
    # Number that will appear on screen
    number = random.randint(1, 9)
    print('    %4d    %4d' % (number, score))

    if random.random() > 0.5:
        print(comments[int(score/100)])
    else:
        if score < 60:
            print('THERE IS NO HOPE')
        elif score > 440:
            print('URK! HELP!!')

    answer = int(input('Enter your guess:'))
    # If you have guessed right increase the score
    if answer == number:
        score = score + number * 2
    else:
        score = score - 10

if score < 0:
    print('YOU ARE NOW MY SLAVE')
else:
    print('OK... you win... (this time)')
